/*---------------------------------------------------------------------------*
 *
 *                  wish - windowing user friendly shell
 *                 --------------------------------------
 *
 *              (c) Copyright Hellmuth Michaelis 1989 - 1993
 *
 *                  Eggerstedtstr. 28
 *                  22765 Hamburg
 *                  Germany
 *
 *                  Tel:    +49 / 40 / 384298    (private)
 *                  Tel:    +49 / 40 / 55903-170 (at work)
 *                  e-mail: hm@hcshh.hcs.de
 *
 *           All rights are reserved except as explicitly granted
 *                  by written permission of the author.
 *
 *             See the file COPYING, distributed with wish, for
 *                  restriction and warranty information
 *
 *---------------------------------------------------------------------------*
 *
 *	last edit-date: [Wed Sep 22 13:27:37 1993]
 *
 *	-hm	primitive filename completition
 *
 *----------------------------------------------------------------------------*/

#include "wish.h"		/* local includes */

static char *wild(char *path, char *wildname);

/*---------------------------------------------------------------------------*
 *	complete name to first matching filename
 *----------------------------------------------------------------------------*/
void complete(void)
{
        char *cptr;		/* ptr to start of line after prompt */
	char *cend;		/* ptr to end of commandline */
	char *p,*q;		/* gp ptr */
	char *csave;		/* filename start position */
	char path[MAXPATHLEN+1];/* path portion */
	char name[MAXPATHLEN+1];/* name portion */
	
	cptr = cbuff+3;			/* init start ptr */
	cend = cptr + strlen(cptr);	/* int end ptr */

	while(cend > cptr)		/* end of commandline */
	{
		cend--;
		if(isspace(*cend))	/* start of filename ? */
			break;
	}
	if(cend <= cptr)		/* hmm ... */
		return;

	cend++;				/* first char of filename */
	csave = cend;			/* save position for later compare */
	
	if((p = strrchr(cend, '/')) == NULL)
	{
		path[0] = '\0';		/* setup path part */
		strcpy(name, cend);	/* setup name part */
	}
	else
	{
		q = path;		/* setup path part */
		while(cend <= p)
			*q++ = *cend++;
		*q = '\0';
		
		q = name;		/* setup name part */
		while(*cend)
			*q++ = *cend++;
		*q = '\0';
	}		
	strcat(name, "*");			/* add wildcard char */
		
	if((q = wild(path, name)) == NULL)	/* resolve wildcards */
		return;				/* not found */

	while(*csave && *q && (*csave++ == *q++))	/* skip same chars */
		;

	while(*q && (cmdline(*q++) == GOOD))	/* rest to commandline */
		;
}
			
/*---------------------------------------------------------------------------*
 *	check if "wildname" matches any name in directory "path"
 *	return concatenated "path" and matched name from dir
 *	if directory found, append a '/' to resulting name
 *----------------------------------------------------------------------------*/
static char *wild(char *path, char *wildname)
{
	register struct direct *dp;		/* dir access */
	register DIR *dirp;			/* dir access */
	struct stat fdbuffer;			/* stat buffer */
	static char wtry[MAXPATHLEN+1];		/* try buffer */
	char rpath[MAXPATHLEN+1];		/* resulting path */
	
	strcpy(rpath, path);			/* path -> result path */
	strcat(rpath, ".");			/* for readdir */
	
	if((dirp = opendir(rpath)) == NULL)
		return(NULL);			/* cannot open directory */

	for(dp = readdir(dirp); dp != NULL; dp = readdir(dirp))
	{
		if((!(strcmp(dp->d_name, "."))) || (!(strcmp(dp->d_name, ".."))))
			continue;

		if(wildmat(dp->d_name, wildname) == TRUE) /* hit ? */
		{
			strcpy(wtry, path);		/* pathname */
			strcat(wtry, dp->d_name);	/* matchname */
			if(stat(wtry, &fdbuffer) < 0)
				continue;		/* not accessible */
			closedir(dirp);			/* close */
			if(S_ISDIR(fdbuffer.st_mode))	/* directory ? */
				strcat(wtry, "/");	/* yes, append / */
			return(wtry);			/* return result */
		}
	}
	closedir(dirp);		/* close */
	return(NULL);		/* failed */
}

/*---------------------------------- EOF -------------------------------------*/
