/*---------------------------------------------------------------------------*
 *
 *                  wish - windowing user friendly shell
 *                 --------------------------------------
 *
 *              (c) Copyright Hellmuth Michaelis 1989 - 1993
 *
 *                  Eggerstedtstr. 28
 *                  22765 Hamburg
 *                  Germany
 *
 *                  Tel:    +49 / 40 / 384298    (private)
 *                  Tel:    +49 / 40 / 55903-170 (at work)
 *                  e-mail: hm@hcshh.hcs.de
 *
 *           All rights are reserved except as explicitly granted
 *                  by written permission of the author.
 *
 *             See the file COPYING, distributed with wish, for
 *                  restriction and warranty information
 *
 *---------------------------------------------------------------------------*
 *
 *	Last Edit-Date: [Mon Aug 30 12:29:19 1993]
 *
 *	-hm	initial configuration menu design
 *	-hm	altcharset debugging
 *	-hm	fkey labels for config screen
 *	-hm	enter wildcard string
 *
 *----------------------------------------------------------------------------*/

#include "wish.h"
#include "config.h"
#include "control.h"

extern char *contin;		/* from help.c */

static char *help1 = "Use cursor keys to move to the items you wish change";
static char *help2 = "SPACE toggles between YES and NO";
static char *help3 = "CONTROL-D exits configuration menu";

static void separate(int);
static char *enter_string(int row, int col, char *string, int length);

static char was_wildcard[OPTSLEN+1];
static char was_wildon;
static char was_cdrom;
static char was_cdnoprog;

/*---------------------------------------------------------------------------*
 *	config function key labels
 *---------------------------------------------------------------------------*/

static struct fk_tab conf_keys[] = {
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"  Exit   Config ", "         "}
};

/*---------------------------------------------------------------------------*
 *	run-time configuration menu
 *---------------------------------------------------------------------------*/
void config(void)
{
	struct item *ip;
	char *q;
	int c;
	int i;
	int dids = 0;
	int helpline = 0;
	
	suspend_time();		/* stop updating time */

	cur_fktab = &conf_keys;	/* config screen fkey label table */
	init_flabels();

	strcpy(was_wildcard, opt_wild);
	was_wildon = opt_wildon;
	was_cdrom = opt_cdrom;
	was_cdnoprog = opt_cdnoprog;
	
	wmove(stdscr, 0, 0);	/* first line */
	wclear(stdscr);		/* clear window */
	touchwin(stdscr);	/* touch it */
	wnoutrefresh(stdscr);	/* force refresh */

	move(0, 0);		/* first line */
	attron(A_REVERSE);	/* highlight on */
	addstr(headerline);	/* program/copyright header */
	attroff(A_REVERSE);	/* highlight off */

	ip = FIRST_I;
	
	/* display menu */
	
	do
	{
		if(!dids && ip->type & ITEM_NOCHANGE)
		{
			dids = 1;
			separate(ip->row - 2);
		}
			
		mvaddstr(ip->row, ip->col, ip->name);
		standout();
		switch(ip->type & ~ITEM_NOCHANGE)
		{
			case ITEM_ONOFF:
				if(*(ip->value))
					addstr("YES");
				else
					addstr(" NO");
				break;
			case ITEM_STRING:
				if(*((char *)ip->value))
					addstr((char *)ip->value);
				else
				{
					standend();
					addstr("[Not set]");
				}
				break;
			case ITEM_CHAR:
				if((char)*(ip->value))
					addch((char)*(ip->value));
				else
				{
					standend();
					addstr("[Not set]");
				}
				break;
		}
		standend();
		if(helpline < ip->row)
			helpline = ip->row;
	}
	while((ip = ip->next) != FIRST_I);

	helpline += 3;

	separate(helpline-1);
	
	move(helpline + 2, (COLS-strlen(help1))/2);
	addstr(help1);

	move(helpline + 3, (COLS-strlen(help2))/2);
	addstr(help2);

	move(helpline + 4, (COLS-strlen(help3))/2);
	addstr(help3);
	
	ip = FIRST_I;

	/* cruise in menu */

	i = 0;
	
	for(;;)
	{
		move(helpline, 0);
		clrtoeol();
		move(helpline, (COLS-strlen(ip->help))/2);
		addstr(ip->help);

		move(ip->row, ((ip->col)+(ip->nl)));
		refresh();

		c = getch();

		switch(c)
		{
			case CR:
				if(ip->type != ITEM_STRING)
					break;

				if((q = enter_string(ip->row, ((ip->col)+(ip->nl)), (char *)ip->value , OPTSLEN)) != NULL)
					strcpy((char *)ip->value, q);

				mvaddstr(ip->row, ip->col, ip->name);
				if(*((char *)ip->value))
				{
					standout();				
					addstr((char *)ip->value);
					standend();
				}
				else
				{
					addstr("[Not set]");
				}
				break;
				
			case ' ':		/* nextopt */
				if(ip->type == ITEM_ONOFF)
				{
					standout();
					if(*(ip->value))
					{
						*(ip->value) = 0;
						addstr(" NO");
					}
					else
					{
						*(ip->value) = 1;
						addstr("YES");
					}
					standend();
				}				
				break;

			case CNTRL_D:
				goto breakout;

			case KEY_UP:		/* up item */
				i = ip->col;
				do
					ip = ip->prev;
				while(ip->col != i);
				break;

			case KEY_DOWN:		/* down item */
				i = ip->col;
				do
					ip = ip->next;
				while(ip->col != i);
				break;

			case KEY_LEFT:		/* left item */
			case KEY_BTAB:
				ip = ip->prev;
				break;

			case KEY_RIGHT:		/* right item */
			case TAB:
				ip = ip->next;
				break;

			case KEY_F(1):	/* function key 1 */
				break;

			case KEY_F(2):	/* function key 2 */
				break;
			
			case KEY_F(3):	/* function key 3 */
				break;
				
			case KEY_F(4):	/* function key 4 */
				break;
				
			case KEY_F(5):	/* function key 5 */
				break;

			case KEY_F(6):	/* function key 6 */
				break;
				
			case KEY_F(7):	/* function key 7 */
				break;
				
			case KEY_F(8):	/* function key 8 */
				goto breakout;

			case CNTRL_L:	/* refresh */
				touchwin(curscr);
				wrefresh(curscr);
				break;
		}
	}

breakout:

	/* do we have to reread dir ? */

	if((strcmp(opt_wild, was_wildcard)) || (opt_wildon != was_wildon) ||
	   (was_cdrom != opt_cdrom) || (was_cdnoprog != opt_cdnoprog))
	{
		free_list();			/* free malloc'ed buffers */
		init_files(PRES_NORM, NULL);	/* yes, read current dir */
	}

	wmove(stdscr, 0, 0);			/* first line */
	wclear(stdscr);				/* clear window */
	touchwin(stdscr);
	wnoutrefresh(stdscr);

	touchwin(cmnd_w);
	touchwin(fst_w);
	touchwin(file_w);
	if(opt_attrib)
		touchwin(attr_w);
	if(opt_labels)
		touchwin(flbl_w);
	
	resume_time();		/* restart updating time */

	header();		/* new header */
	dis_hist();		/* display current commandline */
	fresh_files();		/* refresh files */
	attribs(1);		/* new attributes */
	update_all();		/* update complete screen */

	cur_fktab = &sys_keys;	/* system fkey label table */
	init_flabels();
}

/*---------------------------------------------------------------------------*
 *	print a horizontal line
 *---------------------------------------------------------------------------*/
static void separate(int row)
{
	int i = COLS;
	unsigned char delim;
	
	move(row, 0);

	if(opt_delimiter)
	{
		delim = opt_delimiter;		
	}
	else if(enter_alt_charset_mode && exit_alt_charset_mode &&
	        *enter_alt_charset_mode && *exit_alt_charset_mode)
	{
		attron(A_ALTCHARSET);
		switch(termtype)
		{
			case TERM_HP:	/* HEWLETT-PACKARD Terminals (2392,700/9x etc) */
				delim = ';';
				break;
	
			case TERM_VT2:	/* DEC VT220 / 320 */
			case TERM_VT3:
			case TERM_PCVT:
				delim = 'q';
				break;
	
			default:	/* everything else */
				delim = DEFDELIMCH;
				break;
		}
	}
	else
	{
		delim = DEFDELIMCH;
	}
		
	while(i--)
		addch(delim);

	if(enter_alt_charset_mode && exit_alt_charset_mode &&
           *enter_alt_charset_mode && *exit_alt_charset_mode && !opt_delimiter)
		attroff(A_ALTCHARSET);
}	

/*---------------------------------------------------------------------------*
 *	enter a string into a string variable
 *---------------------------------------------------------------------------*/
static char *enter_string(int row, int col, char *string, int length)
{
	static char buffer[81];
	int ccol, c;

	strcpy(buffer, string);
	ccol = strlen(buffer);

	for(;;)
	{
		mvaddstr(row, col, buffer);
		clrtoeol();
		
		refresh();

		c = getch();	

		if(c == bschar)
		{
			if(ccol > 0)
			{
				ccol--;
				buffer[ccol] = '\0';
			}
			continue;
		}

		switch(c)
		{
			case CR:
				return(buffer);
				
			case ESC:
				return(NULL);
				
			case CNTRL_L:	/* refresh */
				touchwin(curscr);
				wrefresh(curscr);
				break;

			default:
				if(ccol < length)
				{
					buffer[ccol++] = c;
					buffer[ccol] = '\0';
				}
				else
				{
					flash();
				}
				break;
		}
	}
}

/*----------------------- E O F -------------------------------------------*/
