/*---------------------------------------------------------------------------*
 *
 *                  wish - windowing user friendly shell
 *                 --------------------------------------
 *
 *              (c) Copyright Hellmuth Michaelis 1989 - 1993
 *
 *                  Eggerstedtstr. 28
 *                  22765 Hamburg
 *                  Germany
 *
 *                  Tel:    +49 / 40 / 384298    (private)
 *                  Tel:    +49 / 40 / 55903-170 (at work)
 *                  e-mail: hm@hcshh.hcs.de
 *
 *           All rights are reserved except as explicitly granted
 *                  by written permission of the author.
 *
 *             See the file COPYING, distributed with wish, for
 *                  restriction and warranty information
 *
 *---------------------------------------------------------------------------*
 *
 *	Last Edit-Date: [Mon Aug 30 11:34:16 1993]
 *
 *	-hm	conversion to curses
 *	-hm	new sorting order
 *	-hm	displaying file count
 *	-hm	optimizing .....
 *	-hm	wildcard filenames
 *	-hm	cdrom processing option
 *	-hm	cdrom programs to files option
 *
 *----------------------------------------------------------------------------*/

#include "wish.h"		/* global include file */

#ifndef __386BSD__
#include <sys/vfs.h>
#endif
#include <sys/mount.h>

static int  pia[] = {S_IFDIR,S_IREAD,S_IWRITE,S_IEXEC,040,020,010,04,02,01};
static char pca[] = "drwxrwxrwx";

/*---------------------------------------------------------------------------*
 *	create a doubly linked list in sorted order, return pointer to new
 *	first element of list
 *---------------------------------------------------------------------------*/
struct onefile *store
(register struct onefile *new,	/* new entry to store into list */
 register struct onefile *top)	/* current first entry in list */
{
	register struct onefile *old,*p;
	
	if(last == NULL)	/* enter very first element ? */
	{
		new->next = NULL;
		new->prev = NULL;
		last = new;	/* init last */
		return(new);	/* return new first */
	}
	p = top;		/* p = old first element */
	old = NULL;
	while(p)
	{
		if((strcmp(p->onam,new->onam)) < 0)	/* current less new ? */
		{
			old = p;
			p = p->next;
		}
		else
		{	/* current >= new */
		
			if(p->prev)
			{
				p->prev->next = new;
				new->next = p;
				new->prev = p->prev;
				p->prev = new;
				return(top);
			}
			new->next = p;
			new->prev = NULL;
			p->prev = new;
			return(new);
		}
	}
	old->next = new;
	new->next = NULL;
	new->prev = old;
	last = new;
	return(first);
}

/*---------------------------------------------------------------------------*
 *	read current directory and build up a doubly linked sorted list
 *---------------------------------------------------------------------------*/
int fill_list(void)
{
	register struct direct *dp;
	register struct onefile *new_entry;
	register int j;
	register struct tm *tp;
	register DIR *dirp;
	
	static struct stat fdbuffer;
	static char templine[MAXPATHLEN+2];

	int templeng;
	int flcnt = 0;
	int sret;

	maxfnleng = 0;		/* init global variables */	

	iscdfs = 0;		/* no cdrom filesys */
	
	if(opt_cdrom)
	{
		struct statfs statfsb;
#ifndef __386BSD__
	/* the 386BSD way, this also seems to work for SunOS 4.1.x */
		if((statfs(".", &statfsb)==0) && (statfsb.f_fsid[1]==MOUNT_CDFS))
#else
	/* the HPUX way ... */
		if((statfs(".", &statfsb)==0) && (statfsb.f_type==MOUNT_ISOFS))
#endif		
			iscdfs = 1;
	}
		
	if((dirp = opendir(".")) == NULL)
		return(-1);	/* cannot open directory */

	for(dp = readdir(dirp); dp != NULL; dp = readdir(dirp))
	{
  		flcnt++;			/* increment file count */

 		if(!opt_point)			/* display dot ? */
 		{
			if((*(dp->d_name) == '.') && (*(dp->d_name+1) == '\0'))
				continue;
		}

		if(!opt_dotnames)		/* display dot-names ? */
		{
			if((*(dp->d_name) == '.') && (*(dp->d_name+1) != '.') && (*(dp->d_name+1) != '\0'))
				continue;
		}
		
		if((new_entry = (struct onefile *) malloc(sizeof(struct onefile))) == NULL)
		{
			closedir(dirp);		/* clean up */
			return(-2);		/* out of memory */
		}

		new_entry->oislnk = 0;		/* no link yet */
		
		if(opt_links)			/* detect links ? */
		{
			sret = 0;		/* init return */

			if((lstat(dp->d_name, &fdbuffer)) < 0)
			{
				free(new_entry);	/* lstat failed */
				continue;		/* next one .. */
			}

			if(S_ISLNK(fdbuffer.st_mode))	/* have a link ? */
			{
				new_entry->oislnk = 1;	/* mark as link */
				sret = stat(dp->d_name, &fdbuffer); /* and get status */
			}
		}
		else
		{
			sret = stat(dp->d_name, &fdbuffer);	/* get status */
		}

		if(sret < 0)			/* return from stat bad ? */
		{
			free(new_entry);	/* next entry ... */
			continue;
		}
		else	/* got status, fill in from stat buffer */
		{
			/* size, userid & groupid */
			
			new_entry->osiz = fdbuffer.st_size;
			new_entry->ousr = fdbuffer.st_uid;
			new_entry->ogrp = fdbuffer.st_gid;

			/* permissions */
			
			for(j=0;j<10;j++)
			{
				if(fdbuffer.st_mode & pia[j])
					new_entry->oprm[j] = pca[j];
				else
					new_entry->oprm[j] = '-';
			}
			new_entry->oprm[j] = '\0';
			
			/* type of file -> templine[0] */
			
			if(S_ISREG(fdbuffer.st_mode))
			{
				if(opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				new_entry->oprm[0] = '-';
				if((new_entry->oprm[3] == 'x') ||
		                   (new_entry->oprm[6] == 'x') ||
        	        	   (new_entry->oprm[9] == 'x'))
        	        	{
	        	        	if(iscdfs && opt_cdnoprog)
	        	        	{
						new_entry->oprm[3] = '-';
				                new_entry->oprm[6] = '-';
						new_entry->oprm[9] = '-';
						templine[0] = ISDATA;
					}
					else
					{
	        	        	   	templine[0] = ISPROG;
	        	        	}
	        	        }
				else
					templine[0] = ISDATA;
				if((fdbuffer.st_mode & S_IFMT) == S_ISUID)
					new_entry->oprm[3] = 's';
				if((fdbuffer.st_mode & S_IFMT) == S_ISGID)
					new_entry->oprm[6] = 's';
			}
			else if(S_ISDIR(fdbuffer.st_mode))
			{
				if(opt_wildon)
				{
					flcnt--;
				}
				templine[0] = ISDIR;
				new_entry->oprm[0] = 'd';
			}
			else if(S_ISCHR(fdbuffer.st_mode))
			{			
				if(opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISCHAR;
				new_entry->oprm[0] = 'c';
			}
			else if(S_ISBLK(fdbuffer.st_mode))
			{
				if(opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISBLOCK;
				new_entry->oprm[0] = 'b';
			}
			else if(S_ISFIFO(fdbuffer.st_mode))
			{
				if(opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISPIPE;
				new_entry->oprm[0] = 'p';
			}
			else if(S_ISSOCK(fdbuffer.st_mode))
			{
				if(opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISSOCK;
				new_entry->oprm[0] = 'n';
			}
			else
			{
				if(opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISUNKN; /* unknown */
				new_entry->oprm[0] = 'u';
			}
			templine[1] = '\0'; /* terminate templine */

			/* file time */
			
			tp = localtime(&(fdbuffer.st_mtime));
			sprintf(new_entry->odat,"%02d-%02d-%02d %02d:%02d",
				tp->tm_mday,(tp->tm_mon)+1,tp->tm_year,
				tp->tm_hour,tp->tm_min);
			
			new_entry->tag = 0;	/* clear tag-field */
			new_entry->page = -1;	/* page # unknown */

			/* new max filename length */
			
			templeng = strlen(dp->d_name);	/* get name length */
			if(templeng > maxfnleng)	/* new max filename length ? */
				maxfnleng = templeng;	/* yes */

			/* cat templine + filename */

			strcat(templine, dp->d_name);	/* file name */

			strcpy(new_entry->onam, templine);	

			/* sort entry into linked list */
			
			first = store(new_entry,first);
		}
	}
	closedir(dirp);				/* close current dir */
	sprintf(counter,"%4d",flcnt);		/* entries counter */
	return(0);				/* ok return */
}

/*---------------------------------------------------------------------------*
 *	free the current malloc'ed list
 *---------------------------------------------------------------------------*/
void free_list(void)
{
	struct onefile *dir;
	struct onefile *tmp;		

	dir = first;

	while(dir)
	{

		tmp = dir->next;
		free(dir);
		dir = tmp;
	}
	first = NULL;
	last = NULL;
	maxfnleng = 0;
}
 
/*---------------------------------- EOF -------------------------------------*/
