/*---------------------------------------------------------------------------*
 *
 *                  wish - windowing user friendly shell
 *                 --------------------------------------
 *
 *              (c) Copyright Hellmuth Michaelis 1989 - 1993
 *
 *                  Eggerstedtstr. 28
 *                  22765 Hamburg
 *                  Germany
 *
 *                  Tel:    +49 / 40 / 384298    (private)
 *                  Tel:    +49 / 40 / 55903-170 (at work)
 *                  e-mail: hm@hcshh.hcs.de
 *
 *           All rights are reserved except as explicitly granted
 *                  by written permission of the author.
 *
 *             See the file COPYING, distributed with wish, for
 *                  restriction and warranty information
 *
 *---------------------------------------------------------------------------*
 *
 *	last edit-date: [Tue Aug 24 09:50:09 1993]
 *
 *	-hm	modifying function key label layout
 *	-hm	consistent fkey labels ( just 8 ! )
 *	-hm	flabel.c -> fkeys.c
 *	-hm	hpterm support
 *	-hm	interactive configuration
 *	-hm	fkey label structures
 *
 *----------------------------------------------------------------------------*/

#include "wish.h"

/*---------------------------------------------------------------------------*
 *	write string to function key label
 *---------------------------------------------------------------------------*/
void fnclabel(int n, char *string)
{
	char xmit[5];		/* transmitted string ( ESC 'p'...'w' ) */
	char result[50];	/* result buffer */

	/* check if we are on an hp-terminal or lookalike */

	if(termtype == TERM_HP || termtype == TERM_PCVT || termtype == TERM_HPX)
	{
		if(n < 1 || n > 8) /* and it must be a legal key number */
			return;
		if((strlen(string)) != 16)
			return;
			
		xmit[0] = ESC;
		xmit[1] = 'o' + n;
		xmit[2] = '\0';
		sprintf(result,"%c&f0a%dk16d2L%s%s",ESC,n,string,xmit);
		write(2,result,strlen(result));
		sprintf(result,"%c&jB",ESC);	/* display user function keys */
		write(2,result,strlen(result));
		return;
	}
	else if(opt_labels)	/* if virtual f-key labels .. */
	{
		if(n < 1 || n > 8) /* and it must be a legal key number */
			return;

		if((strlen(string)) != 9)
			return;

		wmove(flbl_w, 0, ((n-1)*10)); /* last line */

		wattron(flbl_w, A_REVERSE);
		waddstr(flbl_w, string);
		wattroff(flbl_w, A_REVERSE);
		wrefresh(flbl_w);		
	}
}

/*---------------------------------------------------------------------------*
 *	function key labels initialization
 *---------------------------------------------------------------------------*/
void init_flabels(void)
{
	int i;
	for(i = 0; i < 8; i++)
	{
		if(termtype == TERM_HP || termtype == TERM_PCVT || termtype == TERM_HPX)
			fnclabel(i+1, (*cur_fktab)[i].label_16);
		else if(opt_labels)
			fnclabel(i+1, (*cur_fktab)[i].label_8);
		else
			break;
	}
}

/*---------------------------------------------------------------------------*
 *	restore function key labels
 *---------------------------------------------------------------------------*/
void fini_flabels(void)
{
	static char modes[] = { ESC, '&', 'j', 'A', '\0'};

	 /* check if we are on an hp-terminal */
	
	if(termtype == TERM_HP || termtype == TERM_PCVT || termtype == TERM_HPX)
		write(2,modes,strlen(modes));
}

/*---------------------------------------------------------------------------*
 *	system function keys table
 *---------------------------------------------------------------------------*/

struct fk_tab sys_keys[] = {
	{"  Edit    File  ", "Edit File"},
	{"  Echo  Filename", "Echo Name"},
	{"  Help  Commands", "Help Cmds"},
	{" Config  Screen ", "Conf Scrn"},
	{"  Mark  Filename", "Mark File"},
	{" Unmark allmarkd", "UnmarkAll"},
	{"                ", "         "},
	{" Close   CurDir ", "Close Dir"}
};
	
/*---------------------------------- EOF -------------------------------------*/
