/*---------------------------------------------------------------------------*
 *
 *                  wish - windowing user friendly shell
 *                  ------------------------------------
 *
 *               Copyright (c) 1988-1993 Hellmuth Michaelis
 *
 *                  Eggerstedtstr. 28
 *                  22765 Hamburg
 *                  Germany
 *
 *                  Tel:    +49 / 40 / 384298    (private)
 *                  Tel:    +49 / 40 / 55903-170 (at work)
 *                  e-mail: hm@hcshh.hcs.de
 *
 *                          --------oOo--------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *---------------------------------------------------------------------------*
 *
 *	Last Edit-Date: [Mon Aug 30 19:51:19 1993]
 *
 *	-hm	conversion to curses
 *	-hm	rewrite to not look at filetype anymore
 *	-hm	consistent up/down and left/right moves
 *
 *----------------------------------------------------------------------------*/

#include "wish.h"

static int iscurpag(void);

/*---------------------------------------------------------------------------*
 *	return true if filename is on current displayed page
 *---------------------------------------------------------------------------*/
static int iscurpag(void)
{
	return(((cur_file->page) & PAGEMASK) == cur_page );
}

/*---------------------------------------------------------------------------*
 *	move current file to first file on page
 *---------------------------------------------------------------------------*/
int firstfile(void)
{
	cur_file = scr_beg;

	for(;;)
	{
		if(iscurpag())
			return(GOOD);
		else if(cur_file->next == NULL)
			return(BAD);
		else
			cur_file = cur_file->next;
	}
}

/*---------------------------------------------------------------------------*
 *	move current file to last file on page
 *---------------------------------------------------------------------------*/
int lastfile(void)
{
	cur_file = scr_beg;	/* start of screen */
	
	for(;;)
	{
		if(cur_file->next == NULL)
			break;
		if((cur_file->next->page) == ((cur_page + 1) | FIRST))
			break;
		cur_file = cur_file->next;
	}
	for(;;)
	{
		if(iscurpag())
			return(GOOD);
		else if(cur_file->prev == NULL)
			return(BAD);
		else
			cur_file = cur_file->prev;
	}
}

/*---------------------------------------------------------------------------*
 *	move current file to next file on page
 *---------------------------------------------------------------------------*/
int nextfile(void)
{
	for(;;)
	{
		if(cur_file->next == NULL)
			return(BAD);
		cur_file = cur_file->next;
		if(iscurpag())
			return(GOOD);
	}
}
		
/*---------------------------------------------------------------------------*
 *	move current file to previous file on page
 *---------------------------------------------------------------------------*/
int prevfile(void)
{
	for(;;)
	{
		if(cur_file->prev == NULL)
			return(BAD);
		cur_file = cur_file->prev;
		if(iscurpag())
			return(GOOD);
	}
}

/*---------------------------------------------------------------------------*
 *	move highlighted file to first file on page
 *---------------------------------------------------------------------------*/
void move_home(void)
{
	cur_norm(cur_file);
	firstfile();
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file to last file on page
 *---------------------------------------------------------------------------*/
void move_hmdn(void)
{
	cur_norm(cur_file);
	lastfile();
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file one pos right, wrapping on end
 *---------------------------------------------------------------------------*/
void move_right(void)
{
	cur_norm(cur_file);
	if(nextfile() == BAD)
		firstfile();
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file one pos left, wrapping on begin
 *---------------------------------------------------------------------------*/
void move_left(void)
{
	cur_norm(cur_file);
	if(prevfile() == BAD)
		lastfile();
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file one pos up, wrapping on top
 *---------------------------------------------------------------------------*/
void move_up(void)
{
	struct onefile *save_cur;
	int upcol;
	
	save_cur = cur_file;	/* fail safe */
	
	upcol = cur_file->ocol;	/* current column */
	
	if(first != NULL)
	{
		cur_norm(cur_file);

		while(prevfile() != BAD)
		{
			if(cur_file->ocol == upcol)
				goto upfound;
		}
		if(lastfile() != BAD)
		{
			do
			{
				if(cur_file->ocol == upcol)
					goto upfound;
			}
			while(prevfile() != BAD);
		}
	}
	cur_file = save_cur;	/* not found, no change ! */

upfound:
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file one pos down, wrapping on bottom
 *---------------------------------------------------------------------------*/
void move_down(void)
{
	struct onefile *save_cur;
	int downcol;
	
	save_cur = cur_file;	/* fail safe */
	
	downcol = cur_file->ocol;	/* current column */
	
	if(first != NULL)
	{
		cur_norm(cur_file);

		while(nextfile() != BAD)
		{
			if(cur_file->ocol == downcol)
				goto downfound;
		}
		if(firstfile() != BAD)
		{
			do
			{
				if(cur_file->ocol == downcol)
					goto downfound;
			}
			while(nextfile() != BAD);
		}
	}
	cur_file = save_cur;	/* not found, no change ! */
	
downfound:
	cur_inv(cur_file);
}

/*---------------------------------- EOF -------------------------------------*/

