/*---------------------------------------------------------------------------*
 *
 *                  wish - windowing user friendly shell
 *                  ------------------------------------
 *
 *               Copyright (c) 1988-1993 Hellmuth Michaelis
 *
 *                  Eggerstedtstr. 28
 *                  22765 Hamburg
 *                  Germany
 *
 *                  Tel:    +49 / 40 / 384298    (private)
 *                  Tel:    +49 / 40 / 55903-170 (at work)
 *                  e-mail: hm@hcshh.hcs.de
 *
 *                          --------oOo--------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *---------------------------------------------------------------------------*
 *
 *	Last Edit-Date: [Mon Aug 30 19:55:44 1993]
 *
 *	-hm	conversion to curses
 *	-hm	conversion to multi-windows
 *	-hm	conversion to programs, files & directories
 *	-hm	adding history to commandline
 *	-hm	bsd porting
 *	-hm	preserve dir when cd ..
 *	-hm	stdlib inclusion for malloc()
 *	-hm	getcwd debugging
 *	-hm	fkey handling structures
 *	-hm	wildcard matching cd
 *
 *----------------------------------------------------------------------------*/

#ifdef __386BSD__
#include <ncurses.h>
#else
#include <curses.h>
#endif

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <grp.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <term.h>
#include <time.h>
#include <unistd.h>
#include <utmp.h>
#include <sys/dir.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>

#ifdef MAIN
#define EXTERNAL
#else
#define EXTERNAL extern
#endif

#define EXPSEL	'%'	/* expand selection character in commandline */

#define DEFDELIMCH '-'	/* default char used for horizontal delimiting lines */

/* screen */

#define C_HEAD	 0	/* copyright header line */
#define C_LINE	 1	/* command entry line */
#define C_SEP	 2	/* separator line between command & file display */
#define C_HEIGHT 3	/* commandline height in lines */

#define F_BEG	 0	/* first line of file window */

#define A_SEP	 0	/* separator line between file display & status line */
#define A_LINE	 1	/* attributes line */

#define CURDIRW	11	/* F_CURDIR: current dir string width */
#define ENTRYSW	9	/* F_CURDIR: entries string width */
#define ENTRYCW	4	/* F_CURDIR: entries count width */

/* attribute line */

#define PP_PERM	0		/* position of permission string */

#define PS_SIZE	11		/* position description string */
#define ST_SIZE	"Size:"		/* description */
#define PP_SIZE	PS_SIZE+6	/* position parameter string */

#define PS_USER	27		/* position description string */
#define ST_USER	"User:"		/* description */
#define PP_USER	PS_USER+6	/* position parameter string */

#define PS_GRUP	42		/* position description string */
#define ST_GRUP	"Group:"	/* description */
#define PP_GRUP	PS_GRUP+7	/* position parameter string */

#define PS_DATE	58		/* position description string */
#define ST_DATE	"Date:"		/* description */
#define PP_DATE	PS_DATE+6	/* position parameter string */

#define STOPS 	 30	/* # of positions for filename on screen */
#define MINSPACE 2	/* minimum space between two filenames */
#define STARTCOL 7	/* start column in a row */

#ifndef TRUE
#define TRUE	1	/* TRUE */
#define FALSE	0	/* not TRUE */
#endif

#define ABORT	-1	/* wildmatch */
#define GOOD	TRUE	/* good return */
#define BAD  	FALSE	/* bad return */

#define FIRST	0x100	/* first entry of a file-page */
#define PAGEMASK 0x0ff	/* for reading just the page */

#define CR	0x0d	/* some characters */
#define LF	0x0a
#define TAB	0x09
#define BS	0x08
#define DEL	0x7f
#define SPACE	0x20
#define BEL	0x07
#define ESC	0x1b

/* keycodes for cursor motion */

#define	K_UP	0x10	/* ^P */
#define K_DOWN	0x0e	/* ^N */
#define	K_LEFT	0x02	/* ^B */
#define	K_RIGHT	0x06	/* ^F */

#define K_TAB	0x09	/* tab */

#define K_NEXT	0x16	/* ^V */
#define K_PREV	0x1a	/* ^Z */

#define K_QUIT	0x04	/* ^D */
#define K_REF	0x0c	/* ^L */

#define K_MARK	0x01	/* ^A */
#define	K_ECHO	0x17	/* ^W */
#define K_UMARK	0x15	/* ^U */

#define HISLINES	32	/* no. of lines in history buffer */
#define HISLNLEN	512	/* length of one history line */

/* terminal types */

#define TERM_DUMB		0	/* dumb whatsoever */
#define TERM_HP			1	/* Hewlett Packard 2392 or 700/9x */
#define TERM_VT1		2	/* DEC VT100 */
#define TERM_VT2		3	/* DEC VT220 */
#define TERM_VT3		4	/* DEC VT320 */
#define TERM_PCVT		5	/* 386BSD pcvt driver */
#define TERM_HPX		6	/* hpterm X11 terminal emulator */

/* preserve options */

#define PRES_NO		0		/* don't preserve anything */
#define PRES_NORM	1		/* try "normal" position preserve */
#define PRES_DD		2		/* jump to cur dir after cd .. */

/* one file */

struct onefile {		/* structure for one directory entry */
	char onam[255];		/* filename */
	unsigned int osiz;	/* size */
	int ousr;		/* user */
	int ogrp;		/* group */
	char oislnk;		/* flag, is a link */
	char oprm[16];		/* permissions */
	char odat[15];		/* access date */
	int orow;		/* relative row to display item */
	int ocol;		/* relative column to display item */
	int tag;		/* flag for tagging files */
	int page;		/* page and number for display window */
	struct onefile *next;	/* ptr to next entry */
	struct onefile *prev;	/* prt to previous entry */
};

/* onam[0] contains the entry type, which can be:	*/

#define ISDIR	'1'
#define	ISPROG	'2'
#define	ISDATA	'3'
#define	ISCHAR	'4'
#define	ISBLOCK	'5'
#define	ISPIPE	'6'
#define ISSOCK	'7'
#define ISUNKN	'8'

#define OPTSLEN	60		/* option string length */

#define WILDCHARS "\\?*["	/* special meaning chars */

/* fkey labels */

struct fk_tab {
	char *label_16;		/* HP Terminal Labels */
	char *label_8;		/* DEC (and others) Terminal Labels */
};

extern struct fk_tab sys_keys[];

#ifdef MAIN

char *hislines[HISLINES];	/* array of history line pointers */
int  cur_his = 0;		/* current history line */
char cbuff[HISLNLEN+16];	/* command buffer */

char counter[5] = "\0";		/* entrycount as string */
char cur_path[1024];		/* current path */
char errorline[256];		/* error line if errorflag == 1 */
char headerline[256];		/* headerline for copyright etc */

int stops[STOPS];		/* table of stop points on row */

struct onefile *cur_file = NULL;/* the CURRENT highlighted filename */
struct onefile *first = NULL;	/* init dir-list head-ptr */
struct onefile *last = NULL;	/* init dir-list tail-ptr */

int maxfnleng = 0;		/* length of longest filename of this dir */
int opt_attrib = 1;		/* display attribute line */
int opt_labels = 0;		/* use virtual f-key labels in last line */
int opt_point  = 0;		/* display current dir in file window */
int opt_preserve = 0;		/* initially preserve dirs when cd .. */
int opt_links = 0;		/* initially show files not links */
int opt_return = 1;		/* press any key to continue */
unsigned char opt_delimiter = 0;/* window delimiter character */
int opt_wildon = 0;		/* no wildcarding yet */
char opt_wild[OPTSLEN+1];	/* file window wildcard spec */
char opt_edit[OPTSLEN+1];	/* user sepecified editor */
char opt_more[OPTSLEN+1];	/* user sepecified pager */
int opt_cdrom = 0;		/* cdrom special filename processing */
int opt_cdnoprog = 0;		/* cdrom, convert executables to data */
int opt_dotnames = 1;		/* display filenames starting with dot */

#else /* !MAIN */

extern char *hislines[];
extern int  cur_his;

extern char cbuff[];

extern char counter[];
extern char cur_path[];
extern char errorline[];
extern char headerline[];

extern int stops[];

extern struct onefile *cur_file;
extern struct onefile *first;
extern struct onefile *last;

extern int maxfnleng;
extern int opt_attrib;
extern int opt_labels;
extern int opt_point;
extern int opt_preserve;
extern int opt_links;
extern int opt_return;
extern unsigned char opt_delimiter;
extern opt_wildon;
extern char opt_wild[];
extern char opt_edit[];
extern char opt_more[];
extern int opt_cdrom;
extern int opt_cdnoprog;
extern int opt_dotnames;

#endif /* MAIN */

EXTERNAL char bschar;		/* backspace */

EXTERNAL WINDOW *attr_w;	/* attributes display window (separation,attributes) */
EXTERNAL WINDOW *cmnd_w;	/* command line window (copyright,command,separation) */
EXTERNAL WINDOW *file_w;	/* files display window (file names) */
EXTERNAL WINDOW *fst_w;		/* files status window (cur dir, entries) */
EXTERNAL WINDOW *flbl_w;	/* function key labels */

EXTERNAL char *envhome;		/* ptr to HOME - env var */
EXTERNAL char *envmore;		/* ptr to PAGER - env var */
EXTERNAL char *envedit;		/* ptr to EDITOR - env var */
EXTERNAL char *term_string;	/* terminal type string from environment */

EXTERNAL int termtype;		/* flag what terminal category we are on */

EXTERNAL struct onefile *scr_beg;/* ptr to first filename on screen */
EXTERNAL struct onefile *scr_end;/* ptr to last filename on screen */

EXTERNAL int cur_page;		/* current file window page */
EXTERNAL int errorflag;		/* if error string is in header line */
EXTERNAL int fileheight;	/* height of filewindow */
EXTERNAL struct fk_tab (*cur_fktab)[]; /* current fkey label table */
EXTERNAL int iscdfs;		/* flag, true if we are on a cdrom filesys */

void attribs ( int flag );
void bol_line ( void );
int cd( char *string );
void clear_toeol ( void );
void close_cdir ( void );
void clrerror ( void );
int cmdline ( int c );
void complete ( void );
void config( void );
int cr_on_files ( void );
int curcol ( void );
void cur_blink ( struct onefile *current );
void cur_inv ( struct onefile *current );
void cur_norm ( struct onefile *current );
void del_char ( void );
void dis_hist ( void );
void edit_current ( void );
void eol_line ( void );
void error ( char *str );
void exec_command ( char *cline );
void expsel ( char *p );
void fatal ( char *str );
int fill_list ( void );
void fini_flabels ( void );
int firstfile ( void );
void fnclabel ( int n, char *string );
void free_list ( void );
void fresh_files ( void );
char *group_from_gid ( gid_t gid, int size );
void handlebs ( void );
void handlecr ( void );
void header ( void );
void help ( void );
void h_files ( void );
void h_line ( void );
void init_files ( int preserve, char *dirname );
void init_flabels ( void );
void init_header ( void );
void init_history ( void );
void init_screen ( void );
void init_time ( void );
void is_tagged ( struct onefile *current );
int lastfile ( void );
void left_line ( void );
int main ( int argc, char *argv[] );
void move_down ( void );
void move_hmdn ( void );
void move_home ( void );
void move_left ( void );
void move_right ( void );
void move_up ( void );
void name_echo ( void );
int nextfile ( void );
void next_line ( void );
void next_page ( void );
int prevfile ( void );
void prev_line ( void );
void prev_page ( void );
void readrc ( void );
void resume_time ( void );
void right_line ( void );
void save_line ( void );
void sepaline ( struct _win_st *window );
void set_termtype ( void );
struct onefile *store ( struct onefile *new, struct onefile *top );
void suspend_time ( void );
void tag_current ( struct onefile *current );
void timeout_hdlr ( int sig );
void untag_all ( void );
void update_all ( void );
void update_files ( void );
void usage ( void );
char *user_from_uid ( uid_t uid, int size );
int wildmat( char *text, char *p );
void yank ( void );

/*---------------------------------- EOF -------------------------------------*/
